//
//  PKAppDelegate.m
//  FlowEdit
//
//  Created by Tmi Parabits on 16.7.2013.
//  Copyright (c) 2013 SoftCubicle. All rights reserved.
//

#import "PKAppDelegate.h"

#import "PKViewController.h"
#import "PasteboardCommunicator.h"

@interface PKAppDelegate() {
    
    //__unsafe_unretained PKViewController *_mainViewController;
}

@property (unsafe_unretained) IBOutlet PKViewController *mainViewController;

@end

@implementation PKAppDelegate

@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize managedObjectContext = _managedObjectContext;

- ( void ) onPasteboardCheckTimer: ( NSTimer * ) timer
{
    NSPasteboard * pasteboard = [ NSPasteboard generalPasteboard ];
    NSArray * classes = [ [ NSArray alloc ] initWithObjects: [ NSPasteboardItem class ], nil ];
    
    NSDictionary * options = [ NSDictionary dictionary ];
    NSArray * copiedItems = [ pasteboard readObjectsForClasses: classes
                                                       options: options ];
    
    if( copiedItems.count > 0 )
    {
        NSPasteboardItem * item = [ copiedItems objectAtIndex: 0 ];
        NSData * data = [ item dataForType: [ PasteboardCommunicator UTI ] ];
        
        if( data )
        {
            PasteboardCommunicator * communicator = [ NSKeyedUnarchiver unarchiveObjectWithData: data ];
            
            NSLog(@"Got data.");
            NSLog(@"text = %@", communicator.text);
            
            if( [ communicator.text isEqualToString: @"Liquid Launched" ] )
            {
                /*NSAlert * alert = [ NSAlert alertWithMessageText: @"Liquid Launched"
                                                   defaultButton: @"OK"
                                                 alternateButton: nil
                                                     otherButton: nil
                                       informativeTextWithFormat: @"" ];*/
                //[ alert runModal ];
                
                [ [ PasteboardCommunicator sharedInstance ] writeToPasteboard: @"Author Running" ];
            }
            else if( [ communicator.text isEqualToString: @"Liquid Running" ] )
            {
                [ [ NSPasteboard generalPasteboard ] clearContents ];
            }
            else if( [ communicator.text isEqualToString: @"Liquid Quit" ] )
            {
                [ [ NSPasteboard generalPasteboard ] clearContents ];
                
                /*NSAlert * alert = [ NSAlert alertWithMessageText: @"Liquid Quit"
                                                   defaultButton: @"OK"
                                                 alternateButton: nil
                                                     otherButton: nil
                                       informativeTextWithFormat: @"" ];*/
                //[ alert runModal ];
            } else if ([communicator.text isEqualToString:@"Author Launched"]) {
                [ [ NSPasteboard generalPasteboard ] clearContents ];
            } else if (communicator.attrText) {
                
                
                [ [ NSPasteboard generalPasteboard ] clearContents ];
            } else {
                
                [self.mainViewController pasteLiquidText:communicator.text];
                
                [ [ NSPasteboard generalPasteboard ] clearContents ];
            }
        }
    }
}

- ( void ) applicationWillTerminate:(NSNotification *)notification
{
    [ [ PasteboardCommunicator sharedInstance ] writeToPasteboard: @"Author Quit" ];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    [self.mainViewController loadView];
    
    [ NSTimer scheduledTimerWithTimeInterval: 1.0
                                      target: self
                                    selector: @selector( onPasteboardCheckTimer: )
                                    userInfo: nil
                                     repeats: YES ];
    
    [[ PasteboardCommunicator sharedInstance ] writeToPasteboard: @"Author Launched" ];
}

// Returns the directory the application uses to store the Core Data store file. This code uses a directory named "com.softcubicle.FlowEdit" in the user's Application Support directory.
- (NSURL *)applicationFilesDirectory
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSURL *appSupportURL = [[fileManager URLsForDirectory:NSApplicationSupportDirectory inDomains:NSUserDomainMask] lastObject];
    return [appSupportURL URLByAppendingPathComponent:@"com.softcubicle.FlowEdit"];
}

// Creates if necessary and returns the managed object model for the application.
- (NSManagedObjectModel *)managedObjectModel
{
    if (_managedObjectModel) {
        return _managedObjectModel;
    }
	
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"FlowEdit" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    
    return _managedObjectModel;
}

// Returns the persistent store coordinator for the application. This implementation creates and return a coordinator, having added the store for the application to it. (The directory for the store is created, if necessary.)
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator
{
    if (_persistentStoreCoordinator) {
        return _persistentStoreCoordinator;
    }
    
    NSManagedObjectModel *mom = [self managedObjectModel];
    if (!mom) {
        NSLog(@"%@:%@ No model to generate a store from", [self class], NSStringFromSelector(_cmd));
        return nil;
    }
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSURL *applicationFilesDirectory = [self applicationFilesDirectory];
    NSError *error = nil;
    
    NSDictionary *properties = [applicationFilesDirectory resourceValuesForKeys:@[NSURLIsDirectoryKey] error:&error];
    
    if (!properties) {
        BOOL ok = NO;
        if ([error code] == NSFileReadNoSuchFileError) {
            ok = [fileManager createDirectoryAtPath:[applicationFilesDirectory path] withIntermediateDirectories:YES attributes:nil error:&error];
        }
        if (!ok) {
            [[NSApplication sharedApplication] presentError:error];
            return nil;
        }
    } else {
        if (![properties[NSURLIsDirectoryKey] boolValue]) {
            // Customize and localize this error.
            NSString *failureDescription = [NSString stringWithFormat:@"Expected a folder to store application data, found a file (%@).", [applicationFilesDirectory path]];
            
            NSMutableDictionary *dict = [NSMutableDictionary dictionary];
            [dict setValue:failureDescription forKey:NSLocalizedDescriptionKey];
            error = [NSError errorWithDomain:@"YOUR_ERROR_DOMAIN" code:101 userInfo:dict];
            
            [[NSApplication sharedApplication] presentError:error];
            return nil;
        }
    }
    
    NSURL *url = [applicationFilesDirectory URLByAppendingPathComponent:@"FlowEdit.storedata"];
    NSPersistentStoreCoordinator *coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:mom];
    if (![coordinator addPersistentStoreWithType:NSXMLStoreType configuration:nil URL:url options:nil error:&error]) {
        [[NSApplication sharedApplication] presentError:error];
        return nil;
    }
    _persistentStoreCoordinator = coordinator;
    
    return _persistentStoreCoordinator;
}

// Returns the managed object context for the application (which is already bound to the persistent store coordinator for the application.) 
- (NSManagedObjectContext *)managedObjectContext
{
    if (_managedObjectContext) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (!coordinator) {
        NSMutableDictionary *dict = [NSMutableDictionary dictionary];
        [dict setValue:@"Failed to initialize the store" forKey:NSLocalizedDescriptionKey];
        [dict setValue:@"There was an error building up the data file." forKey:NSLocalizedFailureReasonErrorKey];
        NSError *error = [NSError errorWithDomain:@"YOUR_ERROR_DOMAIN" code:9999 userInfo:dict];
        [[NSApplication sharedApplication] presentError:error];
        return nil;
    }
    _managedObjectContext = [[NSManagedObjectContext alloc] init];
    [_managedObjectContext setPersistentStoreCoordinator:coordinator];

    return _managedObjectContext;
}

// Returns the NSUndoManager for the application. In this case, the manager returned is that of the managed object context for the application.
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window
{
    return [[self managedObjectContext] undoManager];
}

// Performs the save action for the application, which is to send the save: message to the application's managed object context. Any encountered errors are presented to the user.
- (IBAction)saveAction:(id)sender
{
    NSError *error = nil;
    
    if (![[self managedObjectContext] commitEditing]) {
        NSLog(@"%@:%@ unable to commit editing before saving", [self class], NSStringFromSelector(_cmd));
    }
    
    if (![[self managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    // Save changes in the application's managed object context before the application terminates.
    
    if (!_managedObjectContext) {
        return NSTerminateNow;
    }
    
    if (![[self managedObjectContext] commitEditing]) {
        NSLog(@"%@:%@ unable to commit editing to terminate", [self class], NSStringFromSelector(_cmd));
        return NSTerminateCancel;
    }
    
    if (![[self managedObjectContext] hasChanges]) {
        return NSTerminateNow;
    }
    
    NSError *error = nil;
    if (![[self managedObjectContext] save:&error]) {

        // Customize this code block to include application-specific recovery steps.              
        BOOL result = [sender presentError:error];
        if (result) {
            return NSTerminateCancel;
        }

        NSString *question = NSLocalizedString(@"Could not save changes while quitting. Quit anyway?", @"Quit without saves error question message");
        NSString *info = NSLocalizedString(@"Quitting now will lose any changes you have made since the last successful save", @"Quit without saves error question info");
        NSString *quitButton = NSLocalizedString(@"Quit anyway", @"Quit anyway button title");
        NSString *cancelButton = NSLocalizedString(@"Cancel", @"Cancel button title");
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setMessageText:question];
        [alert setInformativeText:info];
        [alert addButtonWithTitle:quitButton];
        [alert addButtonWithTitle:cancelButton];

        NSInteger answer = [alert runModal];
        
        if (answer == NSAlertAlternateReturn) {
            return NSTerminateCancel;
        }
    }

    return NSTerminateNow;
}

@end
