//
//  PKSnippetView.m
//  Liquid Author
//
//  Created by Tmi Parabits on 15.8.2013.
//  Copyright (c) 2013 SoftCubicle. All rights reserved.
//

#import <WebKit/WebKit.h>

#import "PKClippingsView.h"
#import "PKClippingViewController.h"

#import "PKClippingView.h"

#import "PKUtils.h"

@interface PKClippingsView() {
    NSMutableDictionary *clippingViews;
    NSMutableDictionary *clippingsIcons;
    
    NSString *currentDragKey;
}
@end

@implementation PKClippingsView

-(void)mouseMoved:(NSEvent *)theEvent
{
    [PKUtils showLayer:self.layer];
}

#pragma mark Dragging source methods

-(void)mouseDragged:(NSEvent *)theEvent
{
    NSPoint testPoint=[theEvent locationInWindow];
    testPoint=[self convertPoint:testPoint fromView:nil];
    
    for (NSViewController *viewController in [clippingViews allValues]) {
        
        PKClippingView *view=(PKClippingView*)viewController.view;
        
        if (CGRectContainsPoint(view.frame, testPoint)) {
            
            NSPasteboard *pboard=[NSPasteboard pasteboardWithName:NSDragPboard];
            [pboard clearContents];
            
            pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
            [pboard declareTypes:[NSArray arrayWithObject:NSStringPboardType] owner:self];
            
            NSMutableArray *objectsToPasteboard=[[NSMutableArray alloc]init];
            
            /*if (view.attributedString) {
                [objectsToPasteboard addObject:view.attributedString];
            } else {
                
            }*/
            [objectsToPasteboard addObject:view.text];
            
            [pboard writeObjects:objectsToPasteboard];
            
            currentDragKey=view.key;
            
            [self dragImage:[view imageRepresentation] at:testPoint offset:NSMakeSize(0.0, 0.0) event:theEvent pasteboard:pboard source:self slideBack:YES];
            
            break;
        }
    }
}

- (NSDragOperation)draggingSession:(NSDraggingSession *)session sourceOperationMaskForDraggingContext:(NSDraggingContext)context
{
    return NSDragOperationEvery;
}

-(void)draggingSession:(NSDraggingSession *)session endedAtPoint:(NSPoint)screenPoint operation:(NSDragOperation)operation
{
    if (!currentDragKey)return;
    
    NSViewController *viewController=[clippingViews objectForKey:currentDragKey];
    NSView *view=viewController.view;
    
    if (operation!=NSDragOperationNone) {
        
        [view removeFromSuperview];
        
        [clippingViews removeObjectForKey:currentDragKey];
    } else {
        view.alphaValue=1.0;
    }
}

#pragma mark Dragging destination methods

-(NSDragOperation)draggingEntered:(id<NSDraggingInfo>)sender
{
    [PKUtils showLayer:self.layer];
    
    if (!clippingsIcons) {
        clippingsIcons=[[NSMutableDictionary alloc]init];
    }
    if (!clippingViews) {
        clippingViews=[[NSMutableDictionary alloc]init];
    }
    
    NSPasteboard *pasteboard=[sender draggingPasteboard];
    NSArray *items=[pasteboard readObjectsForClasses:@[[NSAttributedString class]] options:nil];
    
    if (!items||items.count==0) {
        return NSDragOperationNone;
    }
    
    NSAttributedString *text=[items objectAtIndex:0];
    NSString *key=[self getKeyForSequenceNumber:sender.draggingSequenceNumber];
    
    PKClippingViewController *clippingview=[[PKClippingViewController alloc]initWithNibName:@"PKClippingViewController" bundle:nil];
    
    [clippingViews setObject:clippingview forKey:key];
    PKClippingView *view=(PKClippingView*)[clippingview view];
    
    [view setKey:key];
    [view setAttributedString:text];
    
    NSFont *font = [NSFont fontWithName:@"Georgia" size:12.0];
    NSDictionary *attrsDictionary =
    [NSDictionary dictionaryWithObject:font
                                forKey:NSFontAttributeName];
    NSAttributedString *attrString =
    [[NSAttributedString alloc] initWithString:text.string
                                    attributes:attrsDictionary];
    
    [[view textView]setAttributedStringValue:attrString];
    
    NSImage *image=[view imageRepresentation];
    [clippingsIcons setObject:image forKey:key];
    
    return NSDragOperationEvery;
}

-(void)updateDraggingItemsForDrag:(id<NSDraggingInfo>)sender
{
    
    if ([[sender draggingSource] isKindOfClass:[PKClippingsView class]])return;
    
    NSString *key=[self getKeyForSequenceNumber:sender.draggingSequenceNumber];
    
    [sender enumerateDraggingItemsWithOptions:NSDraggingItemEnumerationClearNonenumeratedImages forView:self classes:@[[NSString class]] searchOptions:nil usingBlock:^(NSDraggingItem *draggingItem, NSInteger idx, BOOL *stop) {
        draggingItem.imageComponentsProvider=^{
            
            return [self getImageComponentsForKey:key];
        };
    }];
}

-(NSArray*)getImageComponentsForKey:(NSString*)key
{
    NSDraggingImageComponent *imageComponent;
    NSMutableArray *components = [NSMutableArray arrayWithCapacity:1];
    
    NSImage *destination=[clippingsIcons objectForKey:key];
    
    imageComponent = [NSDraggingImageComponent draggingImageComponentWithKey:NSDraggingImageComponentIconKey];
    imageComponent.frame = CGRectMake(0, 0, destination.size.width, destination.size.height);
    imageComponent.contents = destination;
    [components addObject:imageComponent];
    
    return components;
}

-(NSString*)getKeyForSequenceNumber:(NSInteger)number
{
    return [NSString stringWithFormat:@"%ld", (long)number];
}

-(BOOL)performDragOperation:(id<NSDraggingInfo>)sender
{
    NSString *key=[self getKeyForSequenceNumber:sender.draggingSequenceNumber];
    
    PKClippingViewController *controllerView=[clippingViews objectForKey:key];
    PKClippingView *view=(PKClippingView*)controllerView.view;
    
    CGRect rect=view.frame;
    rect.origin=[self convertPoint:sender.draggingLocation fromView:nil];
    rect.origin.y-=view.frame.size.height/2;
    rect.origin.x=15;
    view.frame=rect;
    
    NSString *duplicateTestString=view.text;
    
    for (NSViewController *viewController in [clippingViews allValues]) {
        PKClippingView *possibleDuplicate=(PKClippingView*)viewController.view;
        
        if ([possibleDuplicate.text isEqualToString:duplicateTestString]&& ![possibleDuplicate.key isEqualToString:key]) {
            [possibleDuplicate removeFromSuperview];
            [clippingViews removeObjectForKey:possibleDuplicate.key];
        }
    }
    
    [self addSubview:view];
    
    if ([sender draggingSource]) {
        [[NSNotificationCenter defaultCenter]postNotificationName:@"WebViewCut" object:self];
    }
    
    return YES;
}

@end
