//
//  PKViewController.m
//  FlowEdit
//
//  Created by Tmi Parabits on 17.7.2013.
//  Copyright (c) 2013 SoftCubicle. All rights reserved.
//

#import <WebKit/WebKit.h>
#import <QuartzCore/QuartzCore.h>

#import "PKViewController.h"

#import "PKMainView.h"

#import "PKClippingViewController.h"
#import "PKClippingView.h"

#import "PKWebView.h"

#import "PKClippingsView.h"

#import "PKUtils.h"

//#import "KeyEquivalent.h"

@interface PKViewController () {
    
    NSMutableArray *navigationArray;
    
    int currentNavigationIndex;
    
    NSURL *documentSaveURL;
    NSString *tempFileLoadURL;
    
    BOOL _flowDoc;
    BOOL navigating;
    BOOL firstArrive;
    BOOL openingDocument;
    
    BOOL wordcountOn;
    
    NSString *addLinkText;
}

@property (weak) IBOutlet PKClippingsView *leftClippingsView;
@property (weak) IBOutlet PKClippingsView *rightClippingsView;


@property (strong) NSMutableArray *clippingViews;

@property (weak) IBOutlet NSButton *wordCountButton;
@property (unsafe_unretained) IBOutlet NSWindow *window;

@property (weak) IBOutlet PKMainView *mainView;
@property (weak) IBOutlet NSMenuItem *saveMenuItem;
@property (weak) IBOutlet NSMenu *mainMenu;

@property (strong) IBOutlet NSWindow *openURLSheet;
@property (strong) IBOutlet NSWindow *addLinkSheet;

@property (weak) IBOutlet NSTextField *addLinkTextField;
@property (weak) IBOutlet NSTextField *urlTextField;
@property (weak) IBOutlet PKWebView *myWebView;

@property (weak) IBOutlet NSMenuItem *forwardMenuItem;
@property (weak) IBOutlet NSMenuItem *backMenuItem;

@end

@implementation PKViewController

#pragma mark interface methods
- (IBAction)testClipping:(id)sender {
    
    NSPoint point;
    point.x=0;
    point.y=0;
    
    [self addClipping:point text:@"This is a test."];
}

-(void)addClipping:(NSPoint)location text:(NSString*)text
{
    PKClippingViewController *controller=[[PKClippingViewController alloc]initWithNibName:@"PKClippingViewController" bundle:nil];
    
    NSView *view=controller.view;
    
    CGRect frame=view.frame;
    
    frame.origin.x=50;
    frame.origin.y=location.y;
    
    view.frame=frame;
    
    NSLog(@"Adding %@ at point (%f, %f)", view, location.x, location.y);
    
    [[self clippingViews]addObject:view];
    
    [self.mainView addSubview:view positioned:NSWindowAbove relativeTo:self.myWebView];
    
    for (NSView *view in self.mainView.subviews) {
        NSLog(@"View: %@", view);
    }
}

#pragma mark editor actions

- (IBAction)clickWordCount:(id)sender {

    wordcountOn=!wordcountOn;
    if (wordcountOn) {
        [self countWords];
    } else {
        [self.wordCountButton setTitle:@"Word Count"];
    }
}

- (IBAction)pasteHTML:(id)sender {
    
    NSPasteboard *pasteboard=[NSPasteboard generalPasteboard];
    NSString *data=[pasteboard stringForType:NSStringPboardType];
    
    if (!data)return;
    
    [self.myWebView replaceSelectionWithMarkupString:data];
}

- (IBAction)setLink:(id)sender {
    [NSApp endSheet:self.addLinkSheet];
    
    NSString *address=self.addLinkTextField.stringValue;
    if ([address rangeOfString:@"http://"].location==NSNotFound) {
        address=[NSString stringWithFormat:@"http://%@", address];
    }
    
    NSString *markupString=[NSString stringWithFormat:@"<a href='%@'>%@</a>", address, addLinkText];
    [self.myWebView replaceSelectionWithMarkupString:markupString];
}


- (IBAction)menuAddLink:(id)sender
{    
    DOMRange *selectionRange=[self.myWebView selectedDOMRange];
    addLinkText=selectionRange.text;
    
    NSLog(@"Add Link:%@", addLinkText);
    
    if (!self.addLinkSheet)
        [NSBundle loadNibNamed: @"AddLinkWindow" owner: self];
    
    [self.addLinkSheet orderOut:self];
    [NSApp beginSheet: self.addLinkSheet modalForWindow: self.window modalDelegate: self didEndSelector: @selector(didEndSheet:returnCode:contextInfo:) contextInfo: nil];
}

- (IBAction)cancelAddLink:(id)sender {
    
    [NSApp endSheet:self.addLinkSheet];
}

- (IBAction)showHelp:(id)sender {
    [self loadDocumentWithStringPath:[[NSBundle mainBundle]pathForResource:@"help" ofType:@"html"]];
}

-(void)resolveAndLoadCurrentURL
{
    NSString *loadURL=[navigationArray objectAtIndex:currentNavigationIndex];
    
    NSLog(@"Load url:%@", loadURL);
    
    navigating=YES;
    
    if ([loadURL rangeOfString:@"http://"].location!=NSNotFound) {
        //http
        [[self.myWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:loadURL]]];
    } else {
        //file
        [self loadDocumentWithStringPath:loadURL];
    }
}

-(void)sliceNavigationFromNextPositionToEnd
{
    int location=currentNavigationIndex+1;
    
    if ([navigationArray count]==0) {
        return;
    }
    
    if (location==[navigationArray count]-1) {
        return;
    }
    
    NSRange range;
    
    range.location=currentNavigationIndex+1;
    range.length=[navigationArray count]-range.location;
    
    [navigationArray removeObjectsInRange:range];
}

#pragma mark window delegate functions

- (NSApplicationPresentationOptions)window:(NSWindow *)window willUseFullScreenPresentationOptions:(NSApplicationPresentationOptions)proposedOptions
{
    //[self.toolBar1 setVisible:NO];
    return (NSApplicationPresentationFullScreen |
            NSApplicationPresentationHideDock |
            NSApplicationPresentationAutoHideMenuBar |
            NSApplicationPresentationAutoHideToolbar);
}



-(void)windowWillEnterFullScreen:(NSNotification *)notification
{
    //[self.toolBar1 setVisible:NO];
}

-(void)windowWillExitFullScreen:(NSNotification *)notification
{
    //[self.toolBar1 setVisible:YES];
}

#pragma mark webView dragging delegate functions

-(void)webView:(WebView *)webView willPerformDragDestinationAction:(WebDragDestinationAction)action forDraggingInfo:(id<NSDraggingInfo>)draggingInfo
{
    
}

-(void)webView:(WebView *)webView willPerformDragSourceAction:(WebDragSourceAction)action fromPoint:(NSPoint)point withPasteboard:(NSPasteboard *)pasteboard
{
    
}

#pragma mark webView delegate functions

-(void)countWords
{
    DOMDocument *domDocument=[[self.myWebView mainFrame]DOMDocument];
    DOMHTMLElement *body=[domDocument body];
    
    NSString *bodyText=[body textContent];
    NSSpellChecker *checker=[NSSpellChecker sharedSpellChecker];
    NSInteger count=[checker countWordsInString:bodyText language:checker.language];
    
    if (count<0)count=0;
    
    [self.wordCountButton setTitle:[NSString stringWithFormat:@"%ld", (long)count]];
}

-(void)webViewDidChange:(NSNotification *)notification
{
    if (wordcountOn) {
        
        [self countWords];
        
    }
}

-(void)webView:(WebView *)sender decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id<WebPolicyDecisionListener>)listener
{
    NSNumber *webNavigationType=[actionInformation objectForKey:WebActionNavigationTypeKey];
    
    if (webNavigationType.intValue==WebNavigationTypeLinkClicked) {
        navigating=NO;
        firstArrive=NO;
    }
    
    [listener use];
    
}
-(void)webView:(WebView *)webView didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame
{
    [self.window setTitle:title];
}

-(void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
{
    NSObject *lock;
    
    @synchronized (lock) {
        
        if (navigating||firstArrive) {
            return;
        }
        
        firstArrive=YES;
        
        NSString *currentURL=[[[self.myWebView mainFrame]DOMDocument]URL];
        
        if ([currentURL rangeOfString:@"file:"].location!=NSNotFound) {
            currentURL=tempFileLoadURL;
        }
        
        [self sliceNavigationFromNextPositionToEnd];
        CGRect centerWindowFrame=self.window.frame;
        
        //resolve width
        
        CGFloat w=centerWindowFrame.size.width=[[[self.myWebView mainFrame]frameView ]documentView].frame.size.width;
        CGFloat x=centerWindowFrame.origin.x;
        x-=(w-self.window.frame.size.width)*0.5f;
        
        centerWindowFrame.origin.x=x;
        
        [NSAnimationContext runAnimationGroup:^(NSAnimationContext *context)
         {
             [context setDuration:0.5];
             [[self.window animator] setFrame:centerWindowFrame display:YES];
             
         } completionHandler:^{
             
         }];
        
        [navigationArray addObject:currentURL];
        currentNavigationIndex++;
        
        NSLog(@"Current navigation index=%d", currentNavigationIndex);
        
        for (NSString *u in navigationArray) {
            NSLog(@"%@", u);
        }
        
        if (wordcountOn) {
            [self countWords];
        }
    }
}

#pragma mark ViewController methods & menu functions
- (IBAction)forward:(id)sender {
    if (currentNavigationIndex==[navigationArray count]-1)return;
    
    currentNavigationIndex++;
    [self resolveAndLoadCurrentURL];
}

- (IBAction)back:(id)sender {
    if (currentNavigationIndex==0)return;
    
    currentNavigationIndex--;
    [self resolveAndLoadCurrentURL];
}

-(void)escPressed
{
    [self.window toggleFullScreen:self];
}

-(void)pasteLiquidText:(NSString*)text
{
    [self.myWebView insertText:text];
}

- (IBAction)reloadDocument:(id)sender {
    
    [[self.myWebView mainFrame]reload];
}

- (IBAction)openDocumentFromURL:(id)sender {
    
    documentSaveURL=nil;
    
    if (!self.openURLSheet)
        [NSBundle loadNibNamed: @"Window" owner: self];
    
    [self.openURLSheet orderOut:self];
    [NSApp beginSheet: self.openURLSheet modalForWindow: self.window modalDelegate: self didEndSelector: @selector(didEndSheet:returnCode:contextInfo:) contextInfo: nil];
}

- (IBAction)openURL: (id)sender
{
    [NSApp endSheet:self.openURLSheet];
    [self flowDoc:NO];
    
    NSString *openString=self.urlTextField.stringValue;
    NSRange prefixRange=[openString rangeOfString:@"http://"];
    
    if (prefixRange.location==NSNotFound) {
        openString=[NSString stringWithFormat:@"http://%@", openString];
    }
    
    navigating=firstArrive=NO;
    openingDocument=YES;
    
    [[self.myWebView mainFrame] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:openString]]];
}

- (IBAction)cancelOpenURL:(id)sender {
    
    [NSApp endSheet:self.openURLSheet];
}

- (void)didEndSheet:(NSWindow *)sheet returnCode:(NSInteger)returnCode contextInfo:(void *)contextInfo
{
    [sheet orderOut:self];
}

- (IBAction)openDocument:(id)sender {
    
    NSOpenPanel *openPanel=[NSOpenPanel openPanel];
    [openPanel setAllowedFileTypes:@[@"html"]];
    [openPanel beginSheetModalForWindow:self.window completionHandler:^(NSInteger result) {
        
        if (result==NSFileHandlingPanelOKButton) {
            navigating=NO;
            [self loadDocumentWithURLPath:openPanel.URL];
        }
    }];
}

#pragma mark loading and saving document

-(void)loadDocumentWithURLPath:(NSURL*)urlPath {
    
    if (urlPath.isFileURL) {
        [self loadDocumentWithStringPath:urlPath.path];
    }
}

-(void)loadDocumentWithStringPath:(NSString*)stringPath
{
    firstArrive=NO;
    openingDocument=YES;
    tempFileLoadURL=stringPath;
    
    NSLog(@"Store Absolute string:%@", tempFileLoadURL);
    
    NSURL *bundleURL=[[NSBundle mainBundle]bundleURL];
    NSString *data=[NSString stringWithContentsOfFile:stringPath encoding:NSUTF8StringEncoding error:nil];
    
    [[self.myWebView mainFrame]loadHTMLString:data baseURL:bundleURL];
}

- (IBAction)saveDocument:(id)sender {
    
    if (!_flowDoc)return;
    
    DOMDocument *domDocument=[[self.myWebView mainFrame]DOMDocument];
    NSString *title=domDocument.title;
    NSString *htmlSource=[(DOMHTMLElement *)[[[self.myWebView mainFrame] DOMDocument] documentElement] outerHTML];
    
    if (!documentSaveURL) {
        
        NSSavePanel *savePanel=[NSSavePanel savePanel];
        
        [savePanel setAllowedFileTypes:@[@"html"]];
        [savePanel beginSheetModalForWindow:self.window completionHandler:^(NSInteger result) {
            
            if (!title) {
                NSString *newDocumentName=savePanel.URL.filePathURL.lastPathComponent;
                newDocumentName=[newDocumentName stringByReplacingOccurrencesOfString:@".html" withString:@""];
                [domDocument setTitle:newDocumentName];
                [self.window setTitle:newDocumentName];
            }
            
            NSError *error=nil;
            [htmlSource writeToURL:savePanel.URL atomically:YES encoding:NSUTF8StringEncoding error:&error];
            
            documentSaveURL=savePanel.URL;
            
            if (error) {
                NSLog(@"%@, %@", error.userInfo, error);
            }
        }];
        
    } else {
        NSError *error=nil;
        [htmlSource writeToURL:documentSaveURL atomically:YES encoding:NSUTF8StringEncoding error:&error];
    }
}

- (IBAction)newDocument:(id)sender
{
    NSString *stringPath=[[NSBundle mainBundle]pathForResource:@"template1" ofType:@"html"];
    
    [self loadDocumentWithStringPath:stringPath];
    [self saveDocument:sender];
}

-(void)flowDoc:(BOOL)flag
{
    [self.myWebView setEditable:flag];
    [self.saveMenuItem setHidden:!flag];
    
    if (flag) {
        self.wordCountButton.alphaValue=1.0;
    } else {
        self.wordCountButton.alphaValue=0.0;
    }
    
    _flowDoc=flag;
}

-(void)viewLoaded:(id)sender
{
    [self flowDoc:NO];
    WebFrame *frame=[self.myWebView mainFrame];
    
    DOMDocument *domDocument=[frame DOMDocument];
    DOMNodeList *list=[domDocument getElementsByTagName:@"head"];
    
    DOMNode *head=[list item:0];
    DOMNodeList *childNodes=[head childNodes];
    
    for (int i=0;i<childNodes.length;i++) {
        DOMNode *node=[childNodes item:i];
        
        if ([node.localName isEqualToString:@"meta"]&&node.hasAttributes) {
            DOMNamedNodeMap *map=node.attributes;
            
            for (int j=0;j<map.length;j++) {
                if ([[[map item:0]nodeValue]isEqualToString:@"liquidDoc"]) {
                    
                    [self flowDoc:YES];
                }
            }
        }
    }
    [self.window setTitle:domDocument.title];
}

-(BOOL)webView:(WebView *)webView shouldInsertText:(NSString *)text replacingDOMRange:(DOMRange *)range givenAction:(WebViewInsertAction)action
{
    DOMDocument *document=[[self.myWebView mainFrame]DOMDocument];
    NSString *class=@"style329";
    
    DOMNodeList *list=[document getElementsByClassName:class];
    
    for (int i=0; i<list.length; i++) {
        
        DOMNode *node=[list item:i];
        
        if ([range intersectsNode:node]) {
            return NO;
        }
    }
    return YES;
}

-(void)webViewCut:(id)sender
{
    [self.myWebView cut:sender];
}

-(void)clippingViewVisible:(BOOL)visibility
{
    if (visibility) {
        [PKUtils showLayer:self.leftClippingsView.layer];
        [PKUtils showLayer:self.rightClippingsView.layer];
    } else {
        [PKUtils hideLayer:self.leftClippingsView.layer];
        [PKUtils hideLayer:self.rightClippingsView.layer];
    }
}

-(void)loadView
{
    [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(viewLoaded:) name:WebViewProgressFinishedNotification object:nil];
    
    [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(webViewCut:) name:@"WebViewCut" object:nil];
    
    [self flowDoc:NO];
    
    [self loadDocumentWithStringPath:[[NSBundle mainBundle]pathForResource:@"start" ofType:@"html"]];
    
    [self.mainView setViewController:self];
    [self.mainView setWantsLayer:YES];
    
    [self.forwardMenuItem setKeyEquivalent:@"]"];
    [self.forwardMenuItem setKeyEquivalentModifierMask:NSCommandKeyMask];
    
    [self.backMenuItem setKeyEquivalent:@"["];
    [self.backMenuItem setKeyEquivalentModifierMask:NSCommandKeyMask];
    
    [self.myWebView setApplicationNameForUserAgent:@"Liquid Author"];
    [self.myWebView setFrameLoadDelegate:self];
    [self.myWebView setUIDelegate:self];
    [self.myWebView setPolicyDelegate:self];
    [self.myWebView setEditingDelegate:self];
    [self.myWebView setController:self];
    
    for (NSString *type in self.myWebView.registeredDraggedTypes) {
        NSLog(@"%@", type);
    }
    
    [self.leftClippingsView registerForDraggedTypes:self.myWebView.registeredDraggedTypes];
    [self.rightClippingsView registerForDraggedTypes:self.myWebView.registeredDraggedTypes];
    
    [self.window setDelegate:self];
    
    wordcountOn=NO;
    
    self.clippingViews=[[NSMutableArray alloc]init];
    navigationArray=[[NSMutableArray alloc]init];
    
    navigating=NO;
    currentNavigationIndex=-1;
}


@end
