//
//  PasteboardCommunicator.m
//  Liquid Author Test
//



#import "PasteboardCommunicator.h"


PasteboardCommunicator * pasteboardCommunicator = nil;


@implementation PasteboardCommunicator


@synthesize text = _text;
@synthesize attrText = _attrText;


+ ( NSString * ) UTI
{
    return @"com.PasteboardCommunicator";
}


+ ( PasteboardCommunicator * ) sharedInstance
{
    if( pasteboardCommunicator == nil )
        pasteboardCommunicator = [ [ PasteboardCommunicator alloc ] init ];

    return pasteboardCommunicator;
}


- ( BOOL ) writeToPasteboard: ( NSString * ) string
{
    if( string.length )
    {
        self.text = string;

        NSPasteboardItem * pasteboardItem = [ [ NSPasteboardItem alloc ] init ];

        NSArray * types = [ [ NSArray alloc ] initWithObjects: [ PasteboardCommunicator UTI ], nil ];

        BOOL ok = [ pasteboardItem setDataProvider: self
                                          forTypes: types ];

        if( ok )
        {
            NSPasteboard * pasteboard = [ NSPasteboard generalPasteboard ];
            [ pasteboard clearContents ];
            ok = [ pasteboard writeObjects: [ NSArray arrayWithObject: pasteboardItem ] ];
        }

        if( ok )
        {
//            NSLog( @"Writing is OK" );
            return YES;
        }

    } // if( string.length )

    else
    {
        return NO;
        
    } // else

    return NO;
}


- ( BOOL ) writeAttributedToPasteboard: ( NSAttributedString * ) string
{
    self.text = nil;

    if( string.length )
    {
        self.attrText = string;

        NSPasteboardItem * pasteboardItem = [ [ NSPasteboardItem alloc ] init ];

        NSArray * types = [ [ NSArray alloc ] initWithObjects: [ PasteboardCommunicator UTI ], nil ];

        BOOL ok = [ pasteboardItem setDataProvider: self
                                          forTypes: types ];

        if( ok )
        {
            NSPasteboard * pasteboard = [ NSPasteboard generalPasteboard ];
            [ pasteboard clearContents ];
            ok = [ pasteboard writeObjects: [ NSArray arrayWithObject: pasteboardItem ] ];
        }

        if( ok )
        {
//            NSLog( @"Writing is OK" );
            return YES;
        }

    } // if( string.length )

    else
    {
        return NO;
        
    } // else
    
    return NO;
}


- ( void ) pasteboard: ( NSPasteboard * ) pasteboard
                 item: ( NSPasteboardItem * ) item
   provideDataForType: ( NSString * ) type
{
    NSData * data = [ NSKeyedArchiver archivedDataWithRootObject: self ];

    if( data )
        [ item setData: data forType: type ];
}


#pragma mark - NSCoding protocol

- ( void ) encodeWithCoder: ( NSCoder * ) aCoder
{
    if( self.text )
        [ aCoder encodeObject: self.text
                       forKey: @"text" ];
    else if( self.attrText )
        [ aCoder encodeObject: self.attrText
                       forKey: @"attr_text" ];
}


- ( id ) initWithCoder: ( NSCoder * ) aDecoder
{
    self = [ self init ];

    if( self )
    {
        self.text = [ aDecoder decodeObjectForKey: @"text" ];
        self.attrText = [ aDecoder decodeObjectForKey: @"attr_text" ];
    }

    return self;
}

@end
